#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri May 12 10:04:09 2017

@author: steven
"""

import volmdlr as vm

l=0.1

p1=vm.Point2D((0,0))
p2=vm.Point2D((l,0))
p3=vm.Point2D((l,l))
p4=vm.Point2D((0,l))



l1=vm.LineSegment2D(p1,p2)
l2=vm.LineSegment2D(p2,p3)
l3=vm.LineSegment2D(p3,p4)
l4=vm.LineSegment2D(p4,p1)

p5=vm.Point2D((l/2,l/2))
c1=vm.Circle2D(p5,l/5)

ct1=vm.Contour2D([l4,l3,l2,l1])
ct2=vm.Contour2D([c1])
mesh=vm.Mesh2D([ct1,ct2],{},0.01)

print(mesh.GeoScript('mesh2D.geo'))


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Common abstract primitives
"""

from scipy.optimize import linprog
import math
from numpy import zeros

class RoundedLineSegments:
    def __init__(self, points, radius, line_class, arc_class,
                 closed=False, adapt_radius=False, name=''):

        self.points=points
        self.radius=radius
        self.closed=closed
        self.adapt_radius = adapt_radius
        self.npoints = len(points)
        primitives = self.Primitives(line_class, arc_class)
        
        return primitives
    
    def Primitives(self, line_class, arc_class):
        alpha = {}
        dist = {}
        lines_length = {}
        # Computing optimal radii
        rounded_points_indices = sorted(self.radius.keys())
        groups = []
        arcs = {}
        primitives = []

        
        if self.radius != {}:
            group = [rounded_points_indices[0]]
            _, _, _, dist0, alpha0 = self.ArcFeatures(rounded_points_indices[0])
            dist[rounded_points_indices[0]] = dist0
            alpha[rounded_points_indices[0]] = alpha0
            
            for i in rounded_points_indices[1:]:
                # Computing the arc
                ps2, pi2, pe2, dist2, alpha2 = self.ArcFeatures(i)
                dist[i] = dist2
                alpha[i] = alpha2
                if i-1 in self.radius:
                    p1 = self.points[i-1]
                    p2 = self.points[i]
                    l = (p2 - p1).Norm()
                    lines_length[i-1] = l
                    dist1 = dist[i-1]

                    if dist1 + dist2 <= l:
                        groups.append(group)
                        group = [i]
                    else:
                        if not self.adapt_radius:
                            raise ValueError
                        group.append(i)                
                else:
                    if group != []:
                        groups.append(group)
                    group = [i]
            if group != []:
                groups.append(group)
            if self.adapt_radius:
                if self.closed:
                    if 0 in groups[0]:
                        if self.npoints in groups[-1]:
                            new_group = groups[0] + groups[-1]
                            groups[0] = new_group
                            del groups[-1]
        
                groups2 = []
                ndof = 0
                dof = {}
                neq_ub = 0
                bounds = []
                for group in groups:
                    lg = len(group)
                    if lg == 1:
                        # Single point, reducing its radius by simple computation if needed
                        ipoint = group[0]
                        if self.closed:
                            if ipoint == 0:
                                p1 = self.points[-1]
                                p2 = self.points[0]
                                p3 = self.points[1]
                            elif ipoint == self.npoints-1:
                                p1 = self.points[-2]
                                p2 = self.points[-1]
                                p3 = self.points[0]
                            else:
                                p1 = self.points[ipoint - 1]
                                p2 = self.points[ipoint]
                                p3 = self.points[ipoint +1]
        
                        else:
                            p1 = self.points[ipoint - 1]
                            p2 = self.points[ipoint]
                            p3 = self.points[ipoint +1]
                                
        
                        d1 = p1.PointDistance(p2)
                        d2 = p2.PointDistance(p3)

                        if dist[ipoint] > (min(d1, d2)):
                            self.radius[ipoint] = min(self.radius[ipoint], min(d1, d2) * math.tan(alpha[ipoint]))
        
                    else:
                        # Adding to dof
                        bounds.extend([(0, self.radius[j]/math.tan(alpha[j])) for j in group])
                        dof.update({j: ndof+i for i, j in enumerate(group)})
                        ndof += lg
                        groups2.append(group)
                        neq_ub += lg-1
                
                # Constructing simplex problem
                # Concstructing C
                if ndof > 0:
                    C = zeros(ndof)
                    for j, i in dof.items():
                        C[i] = -math.tan(alpha[j])
                        
                    A_ub = zeros((neq_ub, ndof))
                    b_ub = zeros(neq_ub)
                    ieq_ub = 0
            
                    for group in groups2:                    
                        for ip1, ip2 in zip(group[:-1], group[1:]):
                            A_ub[ieq_ub, dof[ip1]] = 1
                            A_ub[ieq_ub, dof[ip2]] = 1
                            b_ub[ieq_ub] = lines_length[ip1]
                            ieq_ub += 1
                    
                    d = linprog(C, A_ub, b_ub, bounds = bounds)
            
                    for ipoint, dof_point in dof.items():
                        r = d.x[dof_point]*math.tan(alpha[ipoint])
                        if r > 1e-10:
                            self.radius[ipoint] = r
                        else:
                            del self.radius[ipoint]
    
            # Creating geometry
            # Creating arcs
            for ipoint, r in self.radius.items():
                ps, pi, pe, _, _ = self.ArcFeatures(ipoint)
                arcs[ipoint] = arc_class(ps, pi, pe)
        
        # Creating lines
        for iline in range(self.npoints-1):
            if iline in self.radius:
                arc1 = arcs[iline]
                primitives.append(arc1)
                if iline+1 in self.radius:
                    arc2 = arcs[iline+1]
                    if arc1.end != arc2.start:
                        primitives.append(line_class(arc1.end, arc2.start))
                else:
                    if arc1.end != self.points[iline+1]:
                        primitives.append(line_class(arc1.end, self.points[iline+1]))
            else:
                p1 = self.points[iline]
                if iline+1 in self.radius:
                    arc2 = arcs[iline+1]
                    if p1 != arc2.start:
                        primitives.append(line_class(p1, arc2.start))
                else:
                    primitives.append(line_class(p1, self.points[iline+1]))

        if self.closed:
            if self.npoints-1 in self.radius:
                arc1 = arcs[self.npoints-1]
                primitives.append(arc1)
                if 0 in self.radius:
                    arc2 = arcs[0]
                    if arc1.end != arc2.start:
                        primitives.append(line_class(arc1.end, arc2.start))
                else:
                    primitives.append(line_class(arc1.end, self.points[iline+1]))
            else:
                p1 = self.points[self.npoints-1]
                if 0 in self.radius:
                    arc2 = arcs[0]
                    if p1 != arc2.start:
                        primitives.append(line_class(p1, arc2.start))
                else:
                    primitives.append(line_class(p1, self.points[0]))
                    
        return primitives


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Fri Mar 10 17:02:36 2017

@author: steven
"""

import volmdlr as vm
import volmdlr.primitives2D as primitives2D
import time
p1=vm.Point2D((0, 0))
p2=vm.Point2D((1, 0))
p3=vm.Point2D((2, 1))
p4=vm.Point2D((1, 0.5))
p5=vm.Point2D((-0.5, 1))

polygon = vm.Polygon2D([p1, p2, p3, p4, p5])
a = polygon.SelfIntersect()
print(a)
#print(c.Area())
#print(c.SecondMomentArea(p1))

import numpy as npy
npy.seterr(divide='raise')

points_inside=[]
points_outside=[]

for i in range(100):
    pt=vm.Point2D(2*npy.random.random(2) - 0.3)
#    print(p.PointDistance(pt))
    if polygon.PointBelongs(pt):
        points_inside.append(pt)
    else:
        points_outside.append(pt)
 
    
#polygon.MPLPlot()
#points_inside.MPLPlot()
#c1=vm.CompositePrimitive2D([polygon, *points_inside])
#c1.MPLPlot()
f, a = polygon.MPLPlot()
for point in points_inside:
    point.MPLPlot(a, style = 'ob')
for point in points_outside:
    point.MPLPlot(a, style = 'or')
#
#c2=vm.CompositePrimitive2D([polygon, *points_outside])
#c2.MPLPlot()
#
#cog_p = polygon.CenterOfMass()
#c3 = vm.CompositePrimitive2D([polygon, cog_p])
#c3.MPLPlot()

# Speed test
t = time.time()
n = 100000
for i in range(n):
    pt=vm.Point2D(2*npy.random.random(2) - 0.3)
#    print(p.PointDistance(pt))
    polygon.PointBelongs(pt)
t= time.time() - t 
print('time spent: {}s, {}s/eval'.format(t, t/n))   


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Jun 21 16:38:31 2017

@author: steven
"""

import volmdlr as vm
import math

r1 = 1.78*0.5
r2 = r1+0.3

theta1 = 12*2*math.pi/360
theta2 = 33*2*math.pi/360


pm1=vm.Point2D((0,-r1))
pm2=vm.Point2D((0,-r2))
pc=vm.Point2D((0,0))
p1=pm1.Rotation(pc,-theta2)
p2=pm1.Rotation(pc,-theta1)
p3=pm1.Rotation(pc,theta1)
p4=pm1.Rotation(pc,theta2)

p8=pm2.Rotation(pc,-theta2)
p7=pm2.Rotation(pc,-theta1)
p6=pm2.Rotation(pc,theta1)
p5=pm2.Rotation(pc,theta2)

border=vm.Polygon2D([p1,p2,p3,p4,p5,p6,p7,p8], name='border')

ptest=vm.Point2D((-0.08366,-0.91306))

projections = []
for line in border.basis_primitives:
    projections.append(line.PointProjection(ptest))



#print('test: ',border.PointDistance(ptest)+(ptest.vector[1]-p2.vector[1]))
cog_p = border.CenterOfMass()

ctest=vm.CompositePrimitive2D([border,ptest]+projections)
f, a = ctest.MPLPlot()
cog_p.MPLPlot(a, style = 'ob')





#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""

Cython functions

"""
# =============================================================================

cdef (double, double) Csub2D(double u1, double u2,
                             double v1, double v2):
    return (u1-v1, u2-v2)

def sub2D(vector1, vector2):
    return Csub2D(vector1[0], vector1[1],
                  vector2[0], vector2[1])

# =============================================================================

cdef (double, double) Cadd2D(double u1, double u2,
                             double v1, double v2,):
    return (u1+v1, u2+v2)

def add2D(vector1, vector2):
    return Cadd2D(vector1[0], vector1[1],
                  vector2[0], vector2[1])

# =============================================================================

cdef (double, double) Cmul2D(double u1, double u2, double value):
    return (u1*value, u2*value)

def mul2D(vector, value):
    return Cmul2D(vector[0], vector[1], value)

# =============================================================================

cdef double CVector2DDot(double u1, double u2,
                         double v1, double v2):
    return u1*v1 + u2*v2

def Vector2DDot(vector1, vector2):
    return CVector2DDot(vector1[0], vector1[1],
                        vector2[0], vector2[1])

# =============================================================================

cdef double CVector2DNorm(double u1, double u2):
    return (u1*u1 + u2*u2)**0.5

def Vector2DNorm(vector):
    return CVector2DNorm(vector[0], vector[1])

# =============================================================================

cdef (double, double, double) Csub3D(double u1, double u2, double u3,
                                     double v1, double v2, double v3):
    return (u1-v1, u2-v2, u3-v3)

def sub3D(vector1, vector2):
    return Csub3D(vector1[0], vector1[1], vector1[2],
                  vector2[0], vector2[1], vector2[2])

# =============================================================================

cdef (double, double, double) Cadd3D(double u1, double u2, double u3,
                                     double v1, double v2, double v3):
    return (u1+v1, u2+v2, u3+v3)

def add3D(vector1, vector2):
    return Cadd3D(vector1[0], vector1[1], vector1[2],
                  vector2[0], vector2[1], vector2[2])

# =============================================================================

cdef (double, double, double) Cmul3D(double u1, double u2, double u3,
                                     double value):
    return (u1*value, u2*value, u3*value)

def mul3D(vector, value):
    return Cmul3D(vector[0], vector[1], vector[2], value)

# =============================================================================

cdef double CVector3DDot(double u1, double u2, double u3,
                         double v1, double v2, double v3):
    return u1*v1 + u2*v2 + u3*v3

def Vector3DDot(vector1, vector2):
    return CVector3DDot(vector1[0], vector1[1], vector1[2],
                        vector2[0], vector2[1], vector2[2])

# =============================================================================

cdef double CVector3DNorm(double u1, double u2, double u3):
    return (u1*u1 + u2*u2 + u3*u3)**0.5

def Vector3DNorm(vector):
    return CVector3DNorm(vector[0], vector[1], vector[2])

# =============================================================================

def PolygonPointBelongs(point, points):

    cdef int i
    cdef int n = len(points)
    cdef bint inside = False
    cdef float x, y, p1x, p1y, p2x, p2y, xints
    x,y=point
    p1x,p1y = points[0]

    for i in range(n+1):
        p2x, p2y = points[i % n]
        if y > min(p1y,p2y):
            if y <= max(p1y,p2y):
                if x <= max(p1x,p2x):
                    if p1y != p2y:
                        xints = (y-p1y)*(p2x-p1x)/(p2y-p1y)+p1x
                    if p1x == p2x or x <= xints:
                        inside = not inside
        p1x, p1y = p2x, p2y

    return inside

# =============================================================================

cdef (double, (double, double)) CLineSegment2DPointDistance((double, double) p1, (double, double) p2, (double, double) point):
    cdef double t
    cdef (double, double) u, projection

    u = sub2D(p2, p1)
    t = max(0, min(1, Vector2DDot(sub2D(point, p1), u) / Vector2DNorm(u)**2))
    projection = add2D(p1, mul2D(u, t))
    return Vector2DNorm(sub2D(projection, point)), projection

def LineSegment2DPointDistance(points, point):
    return CLineSegment2DPointDistance(tuple(points[0]), tuple(points[1]), tuple(point))



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Sun Mar  5 22:01:35 2017

@author: steven


"""

#import numpy as npy
import volmdlr as vm
import volmdlr.primitives3D as primitives3D
import volmdlr.primitives2D as primitives2D
#import math

p1=vm.Point2D((0, 0))
p2=vm.Point2D((0.1, 0.))
p3=vm.Point2D((0.1, 0.2))
p4=vm.Point2D((0, 0.1))
p5=vm.Point2D((-0.01, 0.05))

#p6=vm.Point2D((0.1,0.3))

l1=primitives2D.RoundedLineSegments2D([p1, p2, p3, p4], {2: 0.01}, False)
l2=vm.Arc2D(p4, p5, p1)
c1=vm.Contour2D([l1, l2])
c2=c1.Rotation(vm.Point2D((0,0)),npy.pi,True)
c1.MPLPlot()
c2.MPLPlot()
c3=vm.Contour2D([c1, c2])
c3.MPLPlot()

po = vm.Point3D((0, 0, 0))
xp = vm.Vector3D((1, 0, 0))
yp = vm.Vector3D((0, 1, 0))


profile = primitives3D.ExtrudedProfile(po, xp, yp, c1, [], vm.Vector3D((0, 0.1, 0.2)))

model = vm.VolumeModel([('', [profile])])

#profile.MPLPlot((0,0,0),(1,0,0),(0,1,0))

#model.MPLPlot()

model.FreeCADExport('extrusion2')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar  8 10:58:19 2017

@author: steven
"""

import volmdlr as vm

p1 = vm.Point2D((1,1.45))
p2 = vm.Point2D((0.4,0.1))
l1 = vm.Line2D(p1, p2)

p3 = vm.Point2D((-1,0))
p4 = vm.Point2D((1,-0.5))
l2 = vm.Line2D(p3, p4)

p5,bl1,bl2 = vm.Point2D.LinesIntersection(l1, l2,True)
p6=vm.Point2D.MiddlePoint(p1,p3)

p7=vm.Point2D.LineProjection(p6, l1)
p8=vm.Point2D.LineProjection(p6, l2)

c=vm.CompositePrimitive2D([l1, l2, p5, p6, p7 ,p8])
c.MPLPlot()



#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Feb 19 12:34:01 2018

@author: Steven Masfaraud masfaraud@dessia.tech
"""

import volmdlr as vm
import volmdlr.primitives3D as p3D

sphere=p3D.Sphere(vm.Point3D((0,0,0)),4)

cylinder=p3D.Cylinder(vm.Point3D((0,0,0)), vm.Vector3D((0,0,1)), 0.5, 10)

cut=p3D.Cut(sphere,cylinder, name='cutted sphere')

model=vm.VolumeModel([('cut', [cut])])

model.FreeCADExport('cut')


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon May  8 15:14:49 2017

@author: steven
"""

import volmdlr as vm
import volmdlr.primitives3D as primitives3D
import math

D=55*1e-3
D1=47.6*1e-3
d=30*1e-3
d1=38.3*1e-3
B=13*1e-3
n_balls=13
D_balls=8*1e-3


y_ball0=(D+d)/4
theta_b=math.acos((y_ball0-d1/2)/D_balls/2)

x=vm.Vector3D((1,0,0))
y=vm.Vector3D((0,1,0))

primitives=[]
center=vm.Point3D((0,0,0))
center_ball1=vm.Point3D((0,y_ball0,0))

for i in range(n_balls):    
    angle=i*2*math.pi/n_balls
    center_ball=center_ball1.Rotation(center,x,angle,True)
#    print(center_ball.vector)
    primitives.append(primitives3D.Sphere(center_ball,D_balls/2,'Ball{}'.format(i+1)))

# inner
pi1=vm.Point2D((-B/2,d/2))
pi2=vm.Point2D((-B/2,d1/2))
pi3=vm.Point2D((-0.5*D_balls*math.sin(theta_b),d1/2))
pi4=vm.Point2D((0,y_ball0-D_balls/2))
pi5=vm.Point2D((0.5*D_balls*math.sin(theta_b),d1/2))
pi6=vm.Point2D((B/2,d1/2))
pi7=vm.Point2D((B/2,d/2))

li1=vm.Line2D(pi1,pi2)
li2=vm.Line2D(pi2,pi3)
ci3=vm.Arc2D(pi3,pi4,pi5)
li4=vm.Line2D(pi5,pi6)
li5=vm.Line2D(pi6,pi7)
li6=vm.Line2D(pi7,pi1)

ci=vm.Contour2D([li1,li2,ci3,li4,li5,li6])
inner=primitives3D.RevolvedProfile(center,x,y,[ci],center,x,math.pi*2,'innerring')
primitives.append(inner)

# outter
po1=vm.Point2D((-B/2,D/2))
po2=vm.Point2D((-B/2,D1/2))
po3=vm.Point2D((-0.5*D_balls*math.sin(theta_b),D1/2))
po4=vm.Point2D((0,y_ball0+D_balls/2))
po5=vm.Point2D((0.5*D_balls*math.sin(theta_b),D1/2))
po6=vm.Point2D((B/2,D1/2))
po7=vm.Point2D((B/2,D/2))

lo1=vm.Line2D(po1,po2)
lo2=vm.Line2D(po2,po3)
co3=vm.Arc2D(po3,po4,po5)
lo4=vm.Line2D(po5,po6)
lo5=vm.Line2D(po6,po7)
lo6=vm.Line2D(po7,po1)

co=vm.Contour2D([lo1,lo2,co3,lo4,lo5,lo6])
outter=primitives3D.RevolvedProfile(center,x,y,[co],center,x,math.pi*2)

primitives.append(outter)


model=vm.VolumeModel(primitives)
resp=model.FreeCADExport('/usr/bin/python','bearing','/usr/lib/freecad/lib/',['stl','fcstd'])
print(resp)


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Wed Mar 14 15:32:37 2018

@author: Steven Masfaraud masfaraud@dessia.tech
"""
import volmdlr as vm
import volmdlr.primitives2D as primitives2D
import numpy as npy

#for i in range(20):
triangle_points=[vm.Point2D(npy.random.random(2)) for i in range(3)]
triangle=vm.Polygon2D(triangle_points)


cog_triangle=triangle.CenterOfMass()
c1 = vm.CompositePrimitive2D([triangle, cog_triangle])
c1.MPLPlot()

print(triangle.Area())

p0=vm.Point2D((-1,0))
p1=vm.Point2D((-npy.cos(npy.pi/4),npy.sin(npy.pi/4)))
p2=vm.Point2D((0,1))

a=vm.Arc2D(p2,p1,p0)
l=vm.LineSegment2D(p2,a.center)
list_node = a.Discretise()

c=vm.Contour2D([a,l] + list_node)
print(c.plot_data())
c2 = vm.CompositePrimitive2D([c])
c2.MPLPlot(style='ob')
print(c.Area())


#!/usr/bin/env python3
# -*- coding: utf-8 -*-
"""
Created on Mon Mar  6 14:51:21 2017

@author: steven
"""

import numpy as npy
import volmdlr as vm
import volmdlr.primitives3D as primitives3D
import volmdlr.primitives2D as primitives2D

import math

r1=0.032
r2=0.04
Z=12
theta=2*math.pi/Z
theta1=0.4*theta
theta2=0.4*theta
theta3=theta-theta1-theta2
r3=0.001
e=0.030
r4=0.015

pc=vm.Point2D((0,0))

p1=vm.Point2D((0,r2))
p2=p1.Rotation(pc,0.5*theta1)
p3=p1.Rotation(pc,theta1)
a1=vm.Arc2D(p1,p2,p3)

p4=vm.Point2D((0,r1))
p4.Rotation(pc,theta1+theta3,False)
p5=p4.Rotation(pc,theta2)
p6=p1.Rotation(pc,theta)

l1=primitives2D.RoundedLineSegments2D([p3,p4,p5,p6],{1:r3,2:r3})

#l1=primitives2D.RoundedLines2D([p1,p2,p3,p4],{0:0.01,2:0.01})
#l2=vm.Circle2D(p5,0.01)
L=[a1,l1]
for i in range(Z-1):
    thetar=(i+1)*theta
    L.append(a1.Rotation(pc,thetar,True))
    L.append(l1.Rotation(pc,thetar,True))
#p7=vm.Point2D((0,r4))
l2=vm.Circle2D(pc,r4)

c1=vm.Contour2D(L)
c2=vm.Contour2D([l2])

po=vm.Point3D((0,0,0))
xp=vm.Vector3D((1,0,0))
yp=vm.Vector3D((0,1,0))



#c1.MPLPlot()
#extr_vect=vm.Vector3D((0,0,e))

profile_straight = primitives3D.ExtrudedProfile(po,xp,yp, c1, [c2], (0,0,e),
                                                name='straight')
#
#model_straight=vm.VolumeModel([profile_straight])

profile_helical = primitives3D.HelicalExtrudedProfile(po, xp, yp, 
                                                    vm.Vector3D((0,0,0)),
                                                    vm.Vector3D((0,0,e)),
                                                    28*3.14/180, c1, [c2],
                                                    name='helical')

model=vm.VolumeModel([('helical', [profile_helical]), ('straight', [profile_straight])])


#resp=model_straight.FreeCADExport('python','gear-straight','/usr/lib/freecad/lib/',['stl','fcstd'])
#print(resp)

resp=model.FreeCADExport('gear')
print(resp)